/*
 * AutomatonIteration.cpp
 *
 *  Created on: 29. 11. 2014
 *	  Author: Tomas Pecka
 */

#include "AutomatonIteration.h"

namespace automaton {

namespace transform {

automaton::Automaton AutomatonIteration::iteration(const automaton::Automaton& automaton) {
	return dispatch(automaton.getData());
}

automaton::NFA < > AutomatonIteration::iteration(const automaton::DFA < > & automaton) {
	automaton::NFA < > res(automaton);

	for(const auto& qf : res.getFinalStates())
		for(const auto& t : res.getTransitionsToState(qf))
			res.addTransition(t.first.first, t.first.second, res.getInitialState());

	label::Label newInitialState = label::createUniqueLabel(automaton.getInitialState(), automaton.getStates());
	res.addState(newInitialState);
	res.setInitialState(newInitialState);
	res.addFinalState(newInitialState);

	for(const auto& t : automaton.getTransitionsFromState(automaton.getInitialState()))
		res.addTransition(newInitialState, t.first.second, t.second);

	return res;
}

auto AutomatonIterationDFA = AutomatonIteration::RegistratorWrapper < automaton::NFA < >, automaton::DFA < > > ( AutomatonIteration::iteration );

automaton::NFA < > AutomatonIteration::iteration(const automaton::NFA < > & automaton) {
	automaton::NFA < > res(automaton);

	for(const auto& qf : res.getFinalStates())
		for(const auto& t : res.getTransitionsToState(qf))
			res.addTransition(t.first.first, t.first.second, res.getInitialState());

	label::Label newInitialState = label::createUniqueLabel(automaton.getInitialState(), automaton.getStates());
	res.addState(newInitialState);
	res.setInitialState(newInitialState);
	res.addFinalState(newInitialState);

	for(const auto& t : automaton.getTransitionsFromState(automaton.getInitialState()))
		for (const label::Label & toState : t.second )
			res.addTransition(newInitialState, t.first.second, toState);

	return res;
}

auto AutomatonIterationNFA = AutomatonIteration::RegistratorWrapper < automaton::NFA < >, automaton::NFA < > > (AutomatonIteration::iteration);

} /* namespace transform */

} /* namespace automaton */
